# Runify

Easily turn any object into a service

## Presentation

This library provides a very simple interface for any 'runnable' object (responds to _run_) to be used as a service.
When included into a class, the *Runify* module adds _startup_, _shutdown_ and _restart_ methods, which take care of wrapping execution of the _run_ method into a thread.

## Installation

### Gemfile
```ruby
gem 'runify'
```

### Terminal
```bash
gem install -V runify
```

## Usage

For any object that features a _run_ method, simply include the *Runify* module into the class.
Instances of this class will respond to the following new methods:

#### *startup*

Calling this method will spawn a new thread around the _run_ method, effectively starting the service in the 'background'.

#### *shutdown*

Calling _shutdown_ will request the service to terminate by setting *@stop* to *true* and wait for the thread to complete (join).
Once this method returns, the service has completely terminated.

#### *restart*

The _restart_ method is nothing more than a shortcut to shutdown and startup:

```ruby
def restart
	shutdown
	startup
end
```

#### The *@stop* variable

Any class that includes the runify module should check the *@stop* instance variable in its _run_ method.
Runify will set @stop to _true_ to indicate execution should end and the _run_ method should return when a shutdown is requested.

A typical _run_ method will usually be implemented as such:

```ruby
def run
	update until @stop
end
```

## Examples

### Runifying a simple service class

```ruby
require 'runify'

# Foo Service
class FooService

	# Runify
	include Runify

	def run
		update until @stop
	end

	def update
		# Business Logic goes here...
	end
end

# Start Foo Service in the background
fs = FooService.new
fs.startup
puts 'Foo Service has started'

# Do something while service is running...

# Restart Foo Service
fs.restart
puts 'Foo Service has restarted'

# Do something while service is running...

# Shutdown Foo Service
fs.shutdown
puts 'Foo Service has terminated'
```

## License

The gem is available as open source under the terms of the [MIT License](http://opensource.org/licenses/MIT).
